'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _immutable = require('immutable');

var _constants = require('../../constants');

var addTagAssociations = function addTagAssociations(state, entityType, entityId, tagAssociations) {
  return state.setIn([entityType, entityId], (0, _immutable.fromJS)(tagAssociations));
};

var removeTagAssociation = function removeTagAssociation(state, entityType, entityId, tagAssociationId) {
  var tags = state.getIn([entityType, entityId]);
  if (tags != null) {
    return state.setIn([entityType, entityId], tags.filterNot(function (t) {
      return t.get('id') === tagAssociationId;
    }));
  } else {
    return state;
  }
};

var setPendingDeletions = function setPendingDeletions(state, entityType, entityId, tagsToDelete) {
  return state.setIn(['pendingDeletion', entityType, entityId], (0, _immutable.fromJS)(tagsToDelete));
};

var defaultState = (0, _immutable.fromJS)({});

exports.default = function () {
  var state = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : defaultState;
  var action = arguments[1];

  switch (action.type) {
    case _constants.ADD_TAG_ASSOCIATIONS:
      return addTagAssociations(state, action.payload.entityType, action.payload.entityId, action.payload.tagAssociations);
    case _constants.REMOVE_TAG_ASSOCIATION:
      return removeTagAssociation(state, action.payload.entityType, action.payload.entityId, action.payload.tagAssociationId);
    case _constants.SET_PENDING_DELETION_TAG_ASSOCIATIONS:
      return setPendingDeletions(state, action.payload.entityType, action.payload.entityId, action.payload.tags);
    default:
      return state;
  }
};